@@ -11,6 +11,8 @@
 #include <unistd.h>
 
 #ifdef _WIN32
+  #define fseek _fseeki64
+  #define ftell _ftelli64
   #define realpath(N,R) _fullpath((R),(N),_MAX_PATH)
   #include <direct.h>
   #ifndef PATH_MAX
@@ -29,60 +31,78 @@
   #define strncasecmp _strnicmp
 #endif
 
+long int unshield_get_path_max(Unshield* unshield)
+{
+#ifdef PATH_MAX
+    return PATH_MAX;
+#else
+    long int path_max = pathconf(unshield->filename_pattern, _PC_PATH_MAX);
+    if (path_max <= 0)
+      path_max = 4096;
+    return path_max;
+#endif
+}
+
+char *unshield_get_base_directory_name(Unshield *unshield) {
+    long int path_max = unshield_get_path_max(unshield);
+    char *p = strrchr(unshield->filename_pattern, '/');
+    char *dirname = malloc(path_max);
+
+    if (p) {
+        strncpy(dirname, unshield->filename_pattern, path_max);
+        if ((unsigned int) (p - unshield->filename_pattern) > path_max) {
+            dirname[path_max - 1] = 0;
+        } else
+            dirname[(p - unshield->filename_pattern)] = 0;
+    } else
+        strcpy(dirname, ".");
+
+    return dirname;
+}
+
+
+static char* get_filename(Unshield* unshield, int index, const char* suffix) {
+    if (unshield && unshield->filename_pattern)
+    {
+        long path_max = unshield_get_path_max(unshield);
+        char* filename  = malloc(path_max);
+
+        if (filename == NULL) {
+            unshield_error("Unable to allocate memory.\n");
+            goto exit;
+        }
+
+        if (snprintf(filename, path_max, unshield->filename_pattern, index, suffix) >= path_max) {
+            unshield_error("Pathname exceeds system limits.\n");
+            goto exit;
+        }
+
+    exit:
+        return filename;
+    }
+
+    return NULL;
+}
+
+
 FILE* unshield_fopen_for_reading(Unshield* unshield, int index, const char* suffix)
 {
   if (unshield && unshield->filename_pattern)
   {
     FILE* result = NULL;
-    char* filename = NULL;
-    char* dirname = NULL;
-    char * p = strrchr(unshield->filename_pattern, '/');
+    char* filename = get_filename(unshield, index, suffix);
+    char* dirname = unshield_get_base_directory_name(unshield);
     const char *q;
     struct dirent *dent = NULL;
     DIR *sourcedir = NULL;
-    long int path_max;
+    long int path_max = unshield_get_path_max(unshield);
 
-    #ifdef PATH_MAX
-    path_max = PATH_MAX;
-    #else
-    path_max = pathconf(unshield->filename_pattern, _PC_PATH_MAX);
-    if (path_max <= 0)
-      path_max = 4096;
-    #endif
-
-    dirname = malloc(path_max);
-    filename = malloc(path_max);
-    if (filename == NULL || dirname == NULL)
-    {
-      unshield_error("Unable to allocate memory.\n");
-      goto exit;
-    }
-
-    if(snprintf(filename, path_max, unshield->filename_pattern, index, suffix)>=path_max)
-    {
-      unshield_error("Pathname exceeds system limits.\n");
-      goto exit;
-    }
     q=strrchr(filename,'/');
     if (q)
       q++;
     else
       q=filename;
 
-    if (p)
-    {
-      strncpy( dirname, unshield->filename_pattern,path_max);
-      if ((unsigned int)(p-unshield->filename_pattern) > path_max)
-      {
-        unshield_trace("WARN: size\n");
-        dirname[path_max-1]=0;
-      }
-      else
-        dirname[(p-unshield->filename_pattern)] = 0;
-    }
-    else
-      strcpy(dirname,".");
-
     sourcedir = opendir(dirname);
     /* Search for the File case independent */
     if (sourcedir)
@@ -127,10 +147,10 @@ exit:
   return NULL;
 }
 
-long unshield_fsize(FILE* file)
+long long unshield_fsize(FILE* file)
 {
-  long result;
-  long previous = ftell(file);
+  long long result;
+  long long previous = ftell(file);
   fseek(file, 0L, SEEK_END);
   result = ftell(file);
   fseek(file, previous, SEEK_SET);
@@ -205,7 +225,7 @@ static const char* unshield_utf16_to_utf8(Header* header, const uint16_t* utf16)
 {
   StringBuffer* string_buffer = unshield_add_string_buffer(header); 
   int length = unshield_strlen_utf16(utf16);
-  int buffer_size = 2 * length + 1;
+  int buffer_size = 3 * length + 1;
   char* target = string_buffer->string = NEW(char, buffer_size);
   ConversionResult result = ConvertUTF16toUTF8(
       (const UTF16**)&utf16, utf16 + length + 1, 
