@@ -1,4 +1,10 @@
 #!/bin/bash
+
+if [ "$(uname)" != "Linux" ] ; then
+    echo "This test can only run on Linux  "
+    exit 0
+fi
+
 set -e
 cd `dirname $0`
 MD5_FILE=`pwd`/`basename $0 .sh`.md5
